import * as F from '../src/form-factory'

describe('Form Factory', () => {
  describe('getErrors', () => {
    it('should return a function', () => {
      expect(typeof F.getErrors()).toBe('function')
    })
    it('should return an empty array when there is no property with given key', () => {
      expect(F.getErrors({})('password')).toEqual([])
    })
    it('should return a value of property with given key from a validation property', () => {
      const errors = ['Is required']
      const model = {
        email: '',
        $: F.createValidationProp({ email: errors }),
      }
      expect(F.getErrors(model)('email')).toEqual(errors)
    })
  })

  describe('createValidationProp', () => {
    it('should return an object', () => {
      expect(typeof F.createValidationProp()).toBe('object')
    })
    it('should return an object with `errors` property', () => {
      expect(F.createValidationProp()).toHaveProperty('errors')
    })
    it('should return an object with `invalidFields` property', () => {
      expect(F.createValidationProp()).toHaveProperty('invalidFields')
    })
    it('should return an object with `isValid` property', () => {
      expect(F.createValidationProp()).toHaveProperty('isValid')
    })
    it('should return an empty object as `errors` property when `errors` argument is omitted', () => {
      expect(F.createValidationProp().errors).toEqual({})
    })
    it('should return `errors` argument as an `errors` property when it\'s declared', () => {
      const errors = { password: [] }
      expect(F.createValidationProp(errors).errors).toBe(errors)
    })
    it('should return an array of `errors` keys which value is not empty as `invalidFields` property', () => {
      const errors = {
        password: [],
        email: ['Is invalid'],
        agreement: ['Is required'],
      }
      expect(F.createValidationProp(errors).invalidFields).toEqual(['email', 'agreement'])
    })
    it('should return true as `isValid` property\'s value if `invalidFields` is an empty array', () => {
      expect(F.createValidationProp().isValid).toBe(true)
      expect(F.createValidationProp({ password: [] }).isValid).toBe(true)
    })
    it('should return false as `isValid` property\'s value if `invalidFields` is not an empty array', () => {
      expect(F.createValidationProp({ password: ['Is required'] }).isValid).toBe(false)
      expect(F.createValidationProp({ name: [], email: ['Is required'] }).isValid).toBe(false)
    })
  })

  describe('createValue', () => {
    it('should return an object', () => {
      expect(typeof F.createValue()).toBe('object')
    })
    it('should return an object with a validation property under `$` key', () => {
      const fields = { password: '', email: 'john@example.com' }
      const errors = { password: ['Is required'], email: [] }
      expect(F.createValue()).toHaveProperty('$')
      expect(F.createValue().$).toEqual(F.createValidationProp())
      expect(F.createValue(fields, F.createValidationProp(errors)).$)
        .toEqual(F.createValidationProp(errors))
    })
    it('should return an object with all properties declared in `fields` argument', () => {
      const fields = { password: '', email: 'john@example.com' }
      expect(F.createValue(fields)).toMatchObject(fields)
    })
  })

  describe('validateAll', () => {
    it('should return an empty array when `validators` argument is not declared', () => {
      expect(F.validateAll()).toEqual([])
    })
    it('should return an array with results of validator functions declared in `validators` argument', () => {
      const validators = [() => ['Is required'], () => [], () => ['Is invalid']]
      expect(F.validateAll(validators)).toEqual(['Is required', 'Is invalid'])
    })
    it('should call validator function with `value` and `data`', () => {
      const fn = jest.fn(() => [])
      const value = 'john@example.com'
      const data = { email: 'john@example.com', name: 'John' }
      F.validateAll([fn], value, data)
      expect(fn).toHaveBeenCalledWith(value, data)
    })
  })

  describe('validate', () => {
    it('should return an empty object when `schema` argument is no declared', () => {
      expect(F.validate()).toEqual({})
    })
    it('should return an object with all keys that are in `schema` object when `fields` is not declared', () => {
      const schema = {
        password: {},
        email: {},
        name: {},
      }
      expect(F.validate({}, schema)).toMatchObject(schema)
    })
    it('should return an object with all keys that are in `schema` object and are declared in `fields` array', () => {
      const schema = {
        password: {},
        email: {},
        name: {},
      }
      expect(Object.keys(F.validate({}, schema, ['password']))).toEqual(['password'])
    })
    it('should return a result of `validateAll` function as a value of each property', () => {
      const data = {
        password: '',
        email: 'john@example.com',
        name: 'John',
      }
      const schema = {
        password: { validators: [() => ['Is invalid'], () => ['Internal error']] },
        email: { validators: [() => ['Is required']] },
        name: {},
      }
      const output = {
        name: [],
        password: ['Is invalid', 'Internal error'],
        email: ['Is required'],
      }
      expect(F.validate(data, schema)).toEqual(output)
    })
  })
})
