import * as V from '../src/validators'

describe('Validators', () => {
  describe('createValidator', () => {
    it('should return a function', () => {
      expect(typeof V.createValidator()).toBe('function')
    })
    it('should return a function that returns function', () => {
      expect(typeof V.createValidator()()).toBe('function')
    })
    it('should return a message wrapped in an array when a given function returns false', () => {
      const output = V.createValidator(() => false)('message')()
      expect(output).toEqual(['message'])
    })
    it('should return an empty array when a given function returns true', () => {
      const output = V.createValidator(() => true)('message')()
      expect(output).toEqual([])
    })
  })

  describe('isEmail', () => {
    it('should test a value against the email regex', () => {
      expect(V.isEmail('john@example.com')).toBe(true)
      expect(V.isEmail('john+123@example.com')).toBe(true)
      expect(V.isEmail('john@example')).toBe(false)
      expect(V.isEmail('john@exampl.e')).toBe(false)
    })
  })

  describe('isNotEmpty', () => {
    it('should return false if value is an empty string or has only whitespaces', () => {
      expect(V.isNotEmpty('')).toBe(false)
      expect(V.isNotEmpty('    \n')).toBe(false)
      expect(V.isNotEmpty('string')).toBe(true)
    })
    it('should return false if value is an empty object', () => {
      expect(V.isNotEmpty({})).toBe(false)
      expect(V.isNotEmpty([])).toBe(false)
      expect(V.isNotEmpty(['1', '2'])).toBe(true)
      expect(V.isNotEmpty({ prop: 1 })).toBe(true)
    })
    it('should return false if value is falsy', () => {
      expect(V.isNotEmpty(undefined)).toBe(false)
      expect(V.isNotEmpty(null)).toBe(false)
      expect(V.isNotEmpty(NaN)).toBe(false)
      expect(V.isNotEmpty(false)).toBe(false)
      expect(V.isNotEmpty(0)).toBe(false)
      expect(V.isNotEmpty(1)).toBe(true)
      expect(V.isNotEmpty(true)).toBe(true)
    })
  })

  describe('testRegex', () => {
    it('should test a value against given regex', () => {
      const regex = /a.*/
      expect(V.testRegex('', { regex })).toBe(false)
      expect(V.testRegex('abc', { regex })).toBe(true)
    })
  })

  describe('isInRange', () => {
    it('should check whether a number is in given range', () => {
      expect(V.isInRange(2, { min: 2, max: 5 })).toBe(true)
      expect(V.isInRange(1, { min: 2, max: 5 })).toBe(false)
      expect(V.isInRange(3)).toBe(true)
      expect(V.isInRange(4, { max: 7 })).toBe(true)
    })
  })

  describe('isLengthInRange', () => {
    it('should check whether a number is in given range', () => {
      expect(V.isLengthInRange('abc', { min: 2, max: 5 })).toBe(true)
      expect(V.isLengthInRange('', { min: 2, max: 5 })).toBe(false)
      expect(V.isLengthInRange('abc')).toBe(true)
      expect(V.isLengthInRange('abcd', { max: 7 })).toBe(true)
    })
  })

  describe('validateExpression', () => {
    it('should return a function', () => {
      expect(typeof V.validate.validateExpression()).toBe('function')
    })
    it('should return a message wrapped in an array when a value doesn\'t pass testing against a supplied regex', () => {
      const validator = V.validate.validateExpression('message', { regex: /a.*/ })
      expect(validator('')).toEqual(['message'])
      expect(validator('1')).toEqual(['message'])
    })
    it('should return an empty array when a value passes testing against a supplied regex', () => {
      const validator = V.validate.validateExpression('message', { regex: /a.*/ })
      expect(validator('abc')).toEqual([])
      expect(validator('a')).toEqual([])
    })
    it('should return an empty array when a regex is not defined', () => {
      const validator = V.validate.validateExpression('message')
      expect(validator('abc')).toEqual([])
    })
  })

  describe('validateRequired', () => {
    it('should return a function', () => {
      expect(typeof V.validate.validateRequired()).toBe('function')
    })
    it('should return a message wrapped in an array when a value is empty', () => {
      const validator = V.validate.validateRequired('message')
      expect(validator('')).toEqual(['message'])
      expect(validator('   ')).toEqual(['message'])
    })
    it('should return an empty array when a value is not empty', () => {
      const validator = V.validate.validateRequired('message')
      expect(validator('abc')).toEqual([])
    })
  })

  describe('validateLength', () => {
    it('should return a function', () => {
      expect(typeof V.validate.validateLength()).toBe('function')
    })
    it('should return a message wrapped in an array when a value\'s length is out of a given range', () => {
      const validator = V.validate.validateLength('message', { min: 2, max: 5 })
      expect(validator('')).toEqual(['message'])
      expect(validator('shouldbeinvalid')).toEqual(['message'])
    })
    it('should return an empty array when a value\'s length is in a given range', () => {
      const validator = V.validate.validateLength('message', { min: 2, max: 5 })
      expect(validator('abc')).toEqual([])
      expect(validator('az')).toEqual([])
    })
  })

  describe('validateRange', () => {
    it('should return a function', () => {
      expect(typeof V.validate.validateRange()).toBe('function')
    })
    it('should return a message wrapped in an array when a value\'s length is out of a given range', () => {
      const validator = V.validate.validateRange('message', { min: 15, max: 20 })
      expect(validator(1)).toEqual(['message'])
      expect(validator(14)).toEqual(['message'])
      expect(validator(21)).toEqual(['message'])
    })
    it('should return an empty array when a value\'s length is in a given range', () => {
      const validator = V.validate.validateRange('message', { min: 15, max: 20 })
      expect(validator(15)).toEqual([])
      expect(validator(17)).toEqual([])
      expect(validator(20)).toEqual([])
    })
  })

  describe('validateEmail', () => {
    it('should return a function', () => {
      expect(typeof V.validate.validateEmail()).toBe('function')
    })
    it('should return a message wrapped in an array when a value doesn\'t pass test against the email regex', () => {
      const validator = V.validate.validateEmail('message')
      expect(validator('')).toEqual(['message'])
      expect(validator('john@example')).toEqual(['message'])
      expect(validator('johndoe')).toEqual(['message'])
    })
    it('return an empty array when a value pass test against the email regex', () => {
      const validator = V.validate.validateEmail('message')
      expect(validator('john@example.io')).toEqual([])
      expect(validator('john@doe.com')).toEqual([])
    })
  })
})
