import {
  reduceObject,
  isEmpty,
  get,
  getNonEmptyKeys,
} from './utils'

/**
 * Gets errors from the validation prop
 *
 * @param {Object} formValue
 * @returns {function}
 */
const getErrors = formValue => key => get(formValue, `$.errors[${key}]`, [])

/**
 * Creates empty errors object
 *
 * @param {Object} schema
 * @returns {Object}
 */
const createEmptyErrors = schema => reduceObject(schema, (props, [key]) => ({
  ...props,
  [key]: [],
}))

/**
 * Creates the validation property
 *
 * @param {Object} [errors={}]
 * @returns {{errors, invalidFields: Array, isValid: boolean}}
 */
const createValidationProp = (errors = {}) => {
  const invalidFields = getNonEmptyKeys(errors)
  return {
    errors,
    invalidFields,
    isValid: isEmpty(invalidFields),
  }
}

/**
 * Creates a value property
 *
 * @param {Object} [fields={}]
 * @param {Object} [validationProp]
 * @returns {Object}
 */
const createValue = (fields = {}, validationProp = createValidationProp()) => ({
  ...fields,
  $: validationProp,
})

/**
 * Executes and merges validators
 *
 * @param {Array} validators
 * @param {*} value
 * @param {Object} data
 * @returns {Array}
 */
const validateAll = (validators = [], value, data) => (
  validators.reduce((errors, validator) => (
    [...errors, ...validator(value, data)]
  ), [])
)

/**
 * Validates a data based on a schema
 *
 * @param {Object} data
 * @param {Object} schema
 * @param {Object} fields
 * @returns {Object}
 */
const validate = (data = {}, schema = {}, fields) => (
  reduceObject(schema, (errors, [key, value]) => {
    if (fields && !fields.includes(key)) return errors
    return { ...errors, [key]: validateAll(value.validators, data[key], data) }
  })
)

export {
  getErrors,
  createEmptyErrors,
  createValidationProp,
  createValue,
  validateAll,
  validate,
}
