const createValidator = (func = () => {}) => (msg, params) => (value, data) => (
  func(value, params, data) ? [] : [msg]
)

const EMAIL_REGEX = /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/

/**
 * Tests a value against the email regex
 *
 * @param {string} value
 * @returns {boolean}
 */
const isEmail = value => EMAIL_REGEX.test(value)
const validateEmail = createValidator(isEmail)

/**
 * Tests a value against a regex
 *
 * @param {string} value
 * @param {RegExp} regex
 * @returns {boolean}
 */
const testRegex = (value, { regex } = {}) => !regex || regex.test(value)
const validateExpression = createValidator(testRegex)


/**
 * Checks if a value is not empty
 *
 * @param {*} value
 * @returns {boolean}
 */
const isNotEmpty = value => (
  Boolean(
    typeof value === 'object' && value !== null
      ? Object.keys(value).length
      : typeof value === 'string'
        ? value.trim()
        : value,
  )
)
const validateRequired = createValidator(isNotEmpty)

/**
 * Tests whether a value is in a range
 *
 * @param {number} value
 * @param {number} min
 * @param {number} max
 * @returns {boolean}
 */
const isInRange = (value, { min = 0, max } = {}) => {
  const castedValue = parseFloat(value)
  return max ? castedValue >= min && castedValue <= max : castedValue >= min
}
const validateRange = createValidator(isInRange)

/**
 * Tests whether a length property is in a range
 *
 * @param {number} length
 * @param {*} args
 * @returns {boolean}
 */
const isLengthInRange = ({ length } = {}, ...args) => isInRange(length, ...args)
const validateLength = createValidator(isLengthInRange)

const validate = {
  validateExpression,
  validateRequired,
  validateLength,
  validateEmail,
  validateRange,
}

export {
  createValidator,
  isEmail,
  testRegex,
  isNotEmpty,
  isInRange,
  isLengthInRange,
  validate,
}
